/**
 * Gaming Tools
 * Copyright (C) 2003  Michael Birken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Michael Birken (o__1@hotmail.com)
 * @version 2.1
 */
package gamingtools;

import java.applet.AudioClip;
import java.io.File;
import java.net.*;

/**
 * <P>The <CODE>AudioSource</CODE> class provides methods for loading music
 * and sound effect files. The following file formats are supported: AIFF-C,
 * AIFF, AU, MIDI, MP3, SND, and WAVE. MP3 files should end in ".mp3" and
 * MIDI files should end in ".mid" or ".midi" for proper
 * identification.</P>
 * <P><I>Note: To play MP3 files, the Java Media Framework must be installed
 * and <CODE>jmf.jar</CODE> must be in the classpath.</I><P>
 * @see gamingtools.SoundEffect
 * @see gamingtools.MidiMusic
 * @see gamingtools.JMFSound
 */
public final class AudioSource {

  /** Location of the audio files relative to the root package. */
  public static final String SOUNDS_DIR = "sounds/";

  /**
   * Loads an audio file with the specified name from the sounds directory.
   * @param fileName Include a path relative to the <CODE>sounds</CODE>
   *   directory.
   * @return The loaded audio.
   */
  public static Audio getAudio(String fileName) {
    return getAudio(fileName, null);
  }

  /**
   * Loads an audio file with the specified name from the <CODE>sounds</CODE>
   * directory and associates it with an <CODE>AudioListener</CODE>.
   * @param fileName Include a path relative to the <CODE>sounds</CODE>
   *   directory.
   * @param audioListener Called back when the sound effect or the music stops
   *   playing.
   * @return The loaded audio.
   */
  public static Audio getAudio(String fileName, AudioListener audioListener) {

    fileName = fileName.trim();

    URL fileURL = ClassLoader.getSystemResource(SOUNDS_DIR + fileName);
    if (fileURL == null)
      throw new RuntimeException(
          "Error locating audio file \"" + fileName + "\".");

    if (fileName.endsWith(".mid") || fileName.endsWith(".midi"))
      return new MidiMusic(fileURL, audioListener);
    else if (fileName.endsWith(".mp3"))
      return new JMFSound(fileURL, audioListener);
    else
      return new SoundEffect(fileURL, audioListener);
  }

  /**
   * This constructor is private to prevent instantiation of this class. All
   * methods and fields are static.
   */
  private AudioSource() {
  }
}