/**
 * Gaming Tools
 * Copyright (C) 2003  Michael Birken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Michael Birken (o__1@hotmail.com)
 * @version 2.1
 */
package gamingtools;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.net.URL;
import java.io.IOException;
import java.io.File;

/**
 * The <CODE>ImageSource</CODE> class provides methods for loading images.
 * All methods of this class are static.
 */
public class ImageSource {

  /** Location of the image files relative to the root package. */
  public static final String IMAGES_DIR = "images/";
  /** Describes the characteristics of the monitor. */
  private static GraphicsConfiguration graphicsConfiguration;

  /**
   * Loads an array of images with the specified names. Each name is formed
   * by concatinating an element of <CODE>names</CODE> to the
   * <CODE>suffix</CODE> (i.e. <CODE>names[i] + suffix</CODE>). The suffix
   * can simply be the file extension.
   * @param names Names of the files without the suffix.
   * @param suffix Examples, ".gif" or "_board3.gif".
   * @return The array of loaded images.
   */
  public static Image[] getImages(String[] names, String suffix) {
    final int SIZE = names.length;
    Image[] images = new Image[SIZE];
    for(int i = 0; i < SIZE; i++)
      images[i] = getImage(names[i] + suffix);
    return images;
  }

  /**
   * Loads an image with the specified name from the images directory.
   * @param fileName Include a path relative to the images directory.
   * @return The loaded image.
   */
  public static Image getImage(String fileName) {
    try {
      if (graphicsConfiguration == null)
        graphicsConfiguration = GraphicsEnvironment
            .getLocalGraphicsEnvironment().getDefaultScreenDevice()
                .getDefaultConfiguration();

      fileName = fileName.trim();

      URL fileURL = ClassLoader.getSystemResource(IMAGES_DIR + fileName);
      if (fileURL == null)
        throw new RuntimeException("Unable to locate file.");

      BufferedImage fileImage = ImageIO.read(fileURL);
      BufferedImage compatibleImage = graphicsConfiguration
          .createCompatibleImage(fileImage.getWidth(), fileImage.getHeight(),
              Transparency.BITMASK);
      Graphics g = null;
      try {
        g = compatibleImage.getGraphics();
        g.drawImage(fileImage, 0, 0, null);
      } finally {
        if (g != null)
          g.dispose();
      }
      fileImage = null;

      return compatibleImage;
    } catch(Throwable e) {
      throw new RuntimeException(
          "Error loading image \"" + fileName + "\".", e);
    }
  }

  /**
   * This constructor is private to prevent instantiation of this class. All
   * methods and fields are static.
   */
  private ImageSource() {
  }
}
