/**
 * Gaming Tools
 * Copyright (C) 2003  Michael Birken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Michael Birken (o__1@hotmail.com)
 * @version 2.1
 */
package gamingtools;

import java.net.*;
import javax.media.*;

/**
 * The <CODE>JMFSound</CODE> class provides methods for loading and playing
 * audio files. The following file formats are supported: AIFF-C, AIFF, AU,
 * MIDI, MP3, SND, and WAVE. When this object is no longer needed, the
 * <CODE>dispose()</CODE> method should be invoked. This class requires
 * <CODE>jmf.jar</CODE> to be in the classpath for compilation.
 */
class JMFSound implements Audio {

  /** Starting position of the media. */
  private static final Time TIME_0 = new Time(0);

  /** For playing and controlling the media. */
  private Player player;
  /** Used to determine when the media stopped playing. */
  private ControllerListener controllerListener;
  /** true only when the music is playing in a loop. */
  private boolean isLooping;
  /** Used to restart the media when it finishes for looping. */
  private final ControllerListener LOOP_LISTENER = new ControllerListener() {
    public void controllerUpdate(ControllerEvent event) {
      if (player == null)
        return;
      if (event.getClass() == EndOfMediaEvent.class) {
        player.stop();
        player.start();
      }
    }
  };
  /** true only after the media has been realized. */
  private boolean realizeComplete;
  /** true only after the media has been prefetched. */
  private boolean prefetchComplete;

  /**
   * Constructs a <CODE>JMFSound</CODE> object out of an audio file.
   * @param fileURL URL to the audio file.
   */
  public JMFSound(URL fileURL) {
    this(fileURL, null);
  }

  /**
   * Constructs a <CODE>JMFSound</CODE> object out of an audio file and
   * associates an <CODE>AudioListener</CODE> with it.
   * @param fileURL URL to the audio file.
   * @param audioListener Called back when the media stops playing.
   */
  public JMFSound(URL fileURL, AudioListener audioListener) {
    try {

      // Load the media and wait for it to be prefetched and realized
      ControllerListener listener = new ControllerListener() {
        public void controllerUpdate(ControllerEvent event) {
          if (event.getClass() == RealizeCompleteEvent.class) {
            realizeComplete = true;
            synchronized(this) {
              notifyAll();
            }
          } else if (event.getClass() == PrefetchCompleteEvent.class) {
            prefetchComplete = true;
            synchronized(this) {
              notifyAll();
            }
          }
        }
      };
      player = Manager.createPlayer(fileURL);
      synchronized(listener) {
        player.addControllerListener(listener);
        player.prefetch();
        player.realize();
        while(!(realizeComplete && prefetchComplete))
          listener.wait();
      }
      player.removeControllerListener(listener);

      setAudioListener(audioListener);
    } catch(Throwable t) {
      throw new RuntimeException("Error loading audio \"" + fileURL + "\".", t);
    }
  }

  /**
   * Starts playing this media. Each time this method is called,
   * the media is restarted from the beginning.
   */
  public void play() {
    if (player != null) {
      stop();
      player.start();
    }
  }

  /**
   * Starts playing this media in a loop.
   */
  public void loop() {
    if (player != null && !isLooping) {
      player.stop();
      player.setMediaTime(TIME_0);
      if (controllerListener != null)
        player.removeControllerListener(controllerListener);
      player.addControllerListener(LOOP_LISTENER);
      isLooping = true;
      player.start();
    }
  }

  /**
   * Stops playing this media.
   */
  public void stop() {
    if (player != null) {
      player.stop();
      player.setMediaTime(TIME_0);
      if (isLooping) {
        isLooping = false;
        player.removeControllerListener(LOOP_LISTENER);
        if (controllerListener != null)
          player.addControllerListener(controllerListener);
      }
    }
  }

  /**
   * Registers an <CODE>AudioListener</CODE> with this <CODE>JMFSound</CODE>
   * object to receive notification of when the media stops.
   * @param audioListener Called back when the media stops.
   */
  public void setAudioListener(final AudioListener audioListener) {
    clearAudioListener();
    if (audioListener != null) {
      controllerListener = new ControllerListener() {
        public void controllerUpdate(ControllerEvent event) {
          if (event.getClass() == EndOfMediaEvent.class)
            audioListener.audioStopped();
        }
      };
      player.addControllerListener(controllerListener);
    }
  }

  /**
   * Unregisters the <CODE>AudioListener</CODE> if there is one. This method is
   * equivalent to passing <CODE>null</CODE> to <CODE>setAudioListener()</CODE>.
   */
  public void clearAudioListener() {
    if (player != null && controllerListener != null && !isLooping)
      player.removeControllerListener(controllerListener);
    controllerListener = null;
  }

  /**
   * <P>Releases any system resources used by this object. A
   * <CODE>JMFSound</CODE> object cannot be used after calling this method.</P>
   * <P>The finalization process of the garbage collector also disposes of
   * the same system resources; however, since the finalization process may
   * not run to completion for a long duration, it is preferable to manually
   * free the resources by invoking this method.</P>
   */
  public void dispose() {
    if (player != null) {
      stop();
      clearAudioListener();
      player.close();
      player = null;
    }
  }
}