/**
 * Gaming Tools
 * Copyright (C) 2003  Michael Birken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Michael Birken (o__1@hotmail.com)
 * @version 2.1
 */
package gamingtools;

import java.applet.AudioClip;
import java.net.*;
import javax.sound.midi.*;
import java.util.*;

/**
 * The <CODE>MidiMusic</CODE> class provides methods for loading and playing
 * MIDI files. When this object is no longer needed, the <CODE>dispose()</CODE>
 * method should be invoked.
 */
class MidiMusic implements Audio {

  /** MIDI constant indicating the end of the music track. */
  private static final int END_OF_TRACK = 47;
  /** Device that plays back the MIDI sequence. */
  private static volatile Sequencer sequencer;
  /** Set of all instances of this object. */
  private static Set midiMusicSet;
  /** Data from the MIDI file. */
  private Sequence sequence;
  /** Registered listener that is called back when sequence finishes. */
  private AudioListener audioListener;
  /** Either loops the sequence or calls back the registered listener. */
  private static volatile AudioListener controlListener;

  /** Used to callback the AudioListener set by the user. */
  private final AudioListener USER_LISTENER = new AudioListener() {
    public void audioStopped() {
      synchronized(MidiMusic.class) {
        if (sequencer == null)
          return;
        if (controlListener != this)
          return;
        AudioListener listener = audioListener;
        if (listener != null)
          listener.audioStopped();
      }
    }
  };

  /** Used to restart the music when it finishes for looping. */
  private static final AudioListener LOOP_LISTENER = new AudioListener() {
    public void audioStopped() {
      synchronized(MidiMusic.class) {
        if (sequencer == null)
          return;
        if (controlListener != this)
          return;
        if (sequencer.isRunning())
          sequencer.stop();
        sequencer.setTickPosition(0);
        sequencer.start();
      }
    }
  };

  /**
   * Invokes the controlListener when the sequence finishes. All MidiMusic
   * instances share this MetaEventListener.
   */
  private static final MetaEventListener META_EVENT_LISTENER
      = new MetaEventListener() {
    public void meta(MetaMessage meta) {
      if (sequencer == null)
        return;
      if (meta.getType() == END_OF_TRACK) {
        AudioListener listener = controlListener;
        if (listener != null)
          listener.audioStopped();
      }
    }
  };

  /**
   * Constructs a <CODE>MidiMusic</CODE> object out of a MIDI file.
   * @param fileURL URL to the MIDI file.
   */
  public MidiMusic(URL fileURL) {
    this(fileURL, null);
  }

  /**
   * Constructs a <CODE>MidiMusic</CODE> object out of a MIDI file and
   * associates an <CODE>AudioListener</CODE> with it.
   * @param fileURL URL to the MIDI file.
   * @param audioListener Called back when the music stops playing.
   */
  public MidiMusic(URL fileURL, AudioListener audioListener) {

    try {
      initSequencer();

      sequence = MidiSystem.getSequence(fileURL);
      setAudioListener(audioListener);
    } catch(Throwable t) {
      throw new RuntimeException("Error loading music \"" + fileURL + "\".", t);
    }
  }

  /**
   * Starts playing this music. Each time this method is called, the
   * music is restarted from the beginning.
   */
  public void play() {
    startSequence(USER_LISTENER);
  }

  /**
   * Starts playing this music in a loop.
   */
  public void loop() {
    startSequence(LOOP_LISTENER);
  }

  /**
   * Plays or loops the sequence.
   * @param listener Specify USER_LISTENER for play() and LOOP_LISTENER for
   *   loop().
   */
  private void startSequence(AudioListener listener) {
    synchronized(MidiMusic.class) {
      if (sequencer == null)
        return;

      try {
        stop();
        sequencer.setSequence(sequence);
        sequencer.setTickPosition(0);

        controlListener = listener;

        sequencer.start();
      } catch(Throwable t) {
        throw new RuntimeException("Unable to play midi sequence.", t);
      }
    }
  }

  /**
   * Stops playing this music.
   */
  public void stop() {
    synchronized(MidiMusic.class) {
      if (sequencer == null)
        return;
      controlListener = null;
      if (sequencer.isRunning())
        sequencer.stop();
    }
  }

  /**
   * Registers an <CODE>AudioListener</CODE> with this <CODE>MidiMusic</CODE>
   * object to receive notification of when the music stops playing.
   * @param audioListener Called back when the music stops playing.
   */
  public void setAudioListener(AudioListener audioListener) {
    this.audioListener = audioListener;
  }

  /**
   * Unregisters the <CODE>AudioListener</CODE> if there is one. This method is
   * equivalent to passing <CODE>null</CODE> to <CODE>setAudioListener()</CODE>.
   */
  public void clearAudioListener() {
    audioListener = null;
  }

  /**
   * <P>Releases any system resources used by this object. A
   * <CODE>MidiMusic</CODE> object cannot be used after calling this
   * method.</P>
   * <P>The finalization process of the garbage collector also disposes of
   * the same system resources; however, since the finalization process may
   * not run to completion for a long duration, it is preferable to manually
   * free the resources by invoking this method.</P>
   */
  public void dispose() {
    synchronized(MidiMusic.class) {
      if (sequencer == null)
        return;

      stop();

      midiMusicSet.remove(this);
      if (midiMusicSet.size() != 0)
        return;

      sequencer.removeMetaEventListener(META_EVENT_LISTENER);
      sequencer.close();
      sequencer = null;
    }
  }

  /**
   * Invokes <CODE>dispose()</CODE> during the finalization process of the
   * garbage collector.
   */
  protected void finalize() throws Throwable {
    dispose();
  }

  /**
   * Obtains the MIDI sequencer shared among all MidiMusic objects. The
   * sequencer is closed and disposed only after all MidiMusic instances
   * are disposed.
   */
  private void initSequencer() {
    synchronized(MidiMusic.class) {

      if (midiMusicSet == null)
        midiMusicSet = Collections.synchronizedSet(new HashSet());
      midiMusicSet.add(this);

      if (sequencer != null)
        return;

      try {
        sequencer = MidiSystem.getSequencer();

        if (sequencer == null)
          throw new RuntimeException(
              "Unable to obtain any midi system sequencer.");

        sequencer.open();
        sequencer.addMetaEventListener(META_EVENT_LISTENER);

        if (!(sequencer instanceof Synthesizer)) {
          Synthesizer synthesizer = MidiSystem.getSynthesizer();
          Receiver synthReceiver = synthesizer.getReceiver();
          Transmitter seqTransmitter = sequencer.getTransmitter();
          seqTransmitter.setReceiver(synthReceiver);
        }
      } catch(Throwable t) {
        throw new RuntimeException(
            "Error initializing midi system sequencer.", t);
      }
    }
  }
}