/**
 * Gaming Tools
 * Copyright (C) 2003  Michael Birken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Michael Birken (o__1@hotmail.com)
 * @version 2.1
 */
package gamingtools;

import java.util.*;

/**
 * The origin of stop watches.
 */
public final class StopWatchSource {

  /**
   * Compares stop watches by resolution.
   */
  private static class StopWatchComparable implements Comparable {

    /** The stop watch to compare. */
    private StopWatch stopWatch;
    /** The resolution of the stop watch to compare. */
    private long resolution;

    /**
     * Constructs this class.
     * @param stopWatch Stop watch to compare.
     */
    public StopWatchComparable(StopWatch stopWatch) {
      this.stopWatch = stopWatch;
      this.resolution = stopWatch.getResolution();
    }

    /**
     * Compares this object with the specified object for order.
     * @param o The object to be compared against.
     * @return <CODE>-1</CODE> if the resolution is less-than the resolution
     *   of <CODE>o</CODE>. <CODE>0</CODE> if the resolutions match. And,
     *   <CODE>1</CODE> otherwise.
     */
    public int compareTo(Object o) {
      StopWatchComparable stopWatchComparable = (StopWatchComparable)o;
      if (resolution < stopWatchComparable.resolution)
        return -1;
      else if (resolution > stopWatchComparable.resolution)
        return 1;
      else
        return 0;
    }

    /**
     * Obtains the stop watch to compare.
     * @return Stop watch to compare.
     */
    public StopWatch getStopWatch() {
      return stopWatch;
    }
  }

  /**
   * Obtains the <CODE>StopWatch</CODE> with the highest resolution available.
   * @return Best <CODE>StopWatch</CODE> available.
   */
  public static StopWatch getStopWatch() {

    List stopWatchList = new ArrayList();

    String osName = System.getProperty("os.name");
    if (osName != null && osName.toUpperCase().indexOf("WINDOWS") >= 0
        && WindowsStopWatch.isAvailable())
      stopWatchList.add(new StopWatchComparable(new WindowsStopWatch()));

    try {
      if (Class.forName("com.sun.j3d.utils.timer.J3DTimer") != null)
        stopWatchList.add(new StopWatchComparable(new J3DStopWatch()));
    } catch(Throwable t) {
    }

    try {
      if (Class.forName("javax.media.SystemTimeBase") != null)
        stopWatchList.add(new StopWatchComparable(new JMFStopWatch()));
    } catch(Throwable t) {
    }

    stopWatchList.add(new StopWatchComparable(new SystemStopWatch()));

    Collections.sort(stopWatchList);

    return ((StopWatchComparable)stopWatchList.get(0)).getStopWatch();
  }

  /**
   * This constructor is private to prevent instantiation of this class. All
   * methods and fields are static.
   */
  private StopWatchSource() {
  }
}