/**
 * Gaming Tools
 * Copyright (C) 2003  Michael Birken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Michael Birken (o__1@hotmail.com)
 * @version 2.1
 */
package gamingtools;

import java.net.URL;
import java.io.*;

/**
 * Used for timing the execution of a code segment. When this class is loaded,
 * a 40KB DLL is copied to the temporary directory defined by the system
 * property <CODE>java.io.tmpdir</CODE> unless it is already located there.
 * @see gamingtools.StopWatchSource
 */
class WindowsStopWatch implements StopWatch {

  /** Name and location of the DLL containing the native timing functions. */
  public static final String STORED_FILE = "native/WindowsStopWatch.dll";
  /** Name of the DLL after it is copied to the temporary directory. */
  public static final String TEMP_FILE = "gamingtools_WindowsStopWatch.dll.tmp";
  /** Duration of a count in nanoseconds. */
  public static final long PERIOD;
  /** Stores the start time. */
  private long counter;

  static {
    File tempFile = null;
    InputStream inputStream = null;
    OutputStream outputStream = null;
    try {
      String tempDir = System.getProperty("java.io.tmpdir");
      if (tempDir == null)
        tempDir = System.getProperty("user.dir");
      tempFile = new File(tempDir + TEMP_FILE);

      if (!tempFile.exists()) {
        outputStream = new FileOutputStream(tempFile);

        URL fileURL = ClassLoader.getSystemResource(STORED_FILE);
        if (fileURL == null)
          throw new RuntimeException(
              "Unable to locate file '" + STORED_FILE + "'");
        inputStream = fileURL.openStream();

        byte[] buffer = new byte[65536];
        int bytesRead = -1;
        while((bytesRead = inputStream.read(buffer)) != -1)
          outputStream.write(buffer, 0, bytesRead);
      }
    } catch(Throwable t) {
      throw new RuntimeException("Unable to load '" + STORED_FILE + "'.", t);
    } finally {
      try {
        if (outputStream != null)
          outputStream.close();
        if (inputStream != null)
          inputStream.close();
      } catch(Throwable t) {
        throw new RuntimeException(t);
      }
    }

    System.load(tempFile.toString());

    PERIOD = 1000000000L / queryPerformanceFrequency();
  }

  /**
   * Determines if the Windows high-resolution performance counter is
   * available in this environment.
   * @return <CODE>true</CODE> if the counter is available.
   */
  public static boolean isAvailable() {
    return queryPerformanceFrequency() != 0;
  }

  /**
   * Retrieves the frequency of the Windows high-resolution performance counter.
   * @return The frequency.
   */
  public static native long queryPerformanceFrequency();

  /**
   * Retrieves the current value of the Windows high-resolution performance
   * counter.
   * @return Value of the counter.
   */
  public static native long queryPerformanceCounter();

  /**
   * Starts timing.
   */
  public void start() {
    counter = queryPerformanceCounter();
  }

  /**
   * Stops timing and returns the elapsed time in nanoseconds
   * (10<SUP>-9</SUP> seconds).
   * @return The duration since <CODE>start()</CODE> was called.
   */
  public long stop() {
    return PERIOD * (queryPerformanceCounter() - counter);
  }

  /**
   * The error expected in the measured elapsed time; meaning, the actual
   * elapsed time is somewhere in the range of the measured elapsed time
   * plus-or-minus this value.
   * @return Error expected in measured elapsed time in nanoseconds
   * (10<SUP>-9</SUP> seconds).
   */
  public long getResolution() {
    long total, count1, count2;

    count1 = queryPerformanceCounter();
    count2 = queryPerformanceCounter();
    while(count1 == count2)
      count2 = queryPerformanceCounter();
    total = PERIOD * (count2 - count1);

    count1 = queryPerformanceCounter();
    count2 = queryPerformanceCounter();
    while(count1 == count2)
      count2 = queryPerformanceCounter();
    total += PERIOD * (count2 - count1);

    count1 = queryPerformanceCounter();
    count2 = queryPerformanceCounter();
    while(count1 == count2)
      count2 = queryPerformanceCounter();
    total += PERIOD * (count2 - count1);

    count1 = queryPerformanceCounter();
    count2 = queryPerformanceCounter();
    while(count1 == count2)
      count2 = queryPerformanceCounter();
    total += PERIOD * (count2 - count1);

    return total / 4;
  }
}